@@ -19,23 +19,17 @@ pub struct LinuxPCIDevice {
     device_id: String,
     device_name: String,
     numa_node: isize,
+    enabled: bool,
+    revision: String,
 }
 
 impl Properties for LinuxPCIDevice {
     fn new(path: &str) -> Self {
         let mut path_vec = [path].to_vec();
 
-        let mut device = LinuxPCIDevice {
-            path: PathBuf::from(path_vec.concat()),
-            address: String::new(),
-            class_id: String::new(),
-            class_name: String::new(),
-            vendor_id: String::new(),
-            vendor_name: String::new(),
-            device_id: String::new(),
-            device_name: String::new(),
-            numa_node: -1,
-        };
+        let mut device: LinuxPCIDevice = Default::default();
+
+        device.set_path(PathBuf::from(path_vec.concat()));
 
         // One of the following two conditions will try to autocomplete the path of the
         // PCI device if the one provided doesn't point to a real path in the filesystem.
@@ -93,6 +87,8 @@ impl Properties for LinuxPCIDevice {
         self.set_device_id();
         self.set_numa_node();
         self.set_class_name();
+        self.set_enabled();
+        self.set_revision();
     }
 
     fn path(&self) -> PathBuf {
@@ -131,6 +127,14 @@ impl Properties for LinuxPCIDevice {
         self.device_name.to_owned()
     }
 
+    fn enabled(&self) -> bool {
+        self.enabled
+    }
+
+    fn revision(&self) -> String {
+        self.revision.to_owned()
+    }
+
     fn set_path(&mut self, p: PathBuf) {
         self.path = p;
     }
@@ -146,7 +150,8 @@ impl Properties for LinuxPCIDevice {
     }
 
     fn set_class_id(&mut self) {
-        if let Ok(mut str) = std::fs::read_to_string(&self.path().join("class")) {
+        if let Ok(mut str) = std::fs::read_to_string(&self.path.join("class")) {
+            // This file is guaranteed to end with an EOL character, so let's remove that.
             str.pop();
             str = str.trim_start_matches("0x").chars().take(2).collect();
             self.class_id = str;
@@ -154,32 +159,43 @@ impl Properties for LinuxPCIDevice {
     }
 
     fn set_vendor_id(&mut self) {
-        if let Ok(mut str) = std::fs::read_to_string(&self.path().join("vendor")) {
-            str = str.trim_start_matches("0x").to_string();
+        if let Ok(mut str) = std::fs::read_to_string(&self.path.join("vendor")) {
+            // This file is guaranteed to end with an EOL character, so let's remove that.
             str.pop();
+            str = str.trim_start_matches("0x").to_string();
             self.vendor_id = str;
         }
     }
 
     fn set_device_id(&mut self) {
-        if let Ok(mut str) = std::fs::read_to_string(&self.path().join("device")) {
-            str = str.trim_start_matches("0x").to_string();
+        if let Ok(mut str) = std::fs::read_to_string(&self.path.join("device")) {
+            // This file is guaranteed to end with an EOL character, so let's remove that.
             str.pop();
+            str = str.trim_start_matches("0x").to_string();
             self.device_id = str;
         }
     }
 
+    fn set_revision(&mut self) {
+        if let Ok(mut str) = std::fs::read_to_string(&self.path.join("revision")) {
+            // This file is guaranteed to end with an EOL character, so let's remove that.
+            str.pop();
+            str = str.trim_start_matches("0x").to_string();
+            self.revision = str;
+        }
+    }
+
     fn set_numa_node(&mut self) {
-        if let Ok(v) = std::fs::read_to_string(&self.path().join("numa_node")) {
-            if let Ok(p) = v.parse::<isize>() {
-                return self.numa_node = p;
+        if let Ok(str) = std::fs::read_to_string(&self.path.join("numa_node")) {
+            if let Ok(val) = str.parse::<isize>() {
+                self.numa_node = val;
             }
         }
     }
 
-    /// This function sets the PCI device's `class_name` associated
-    /// with its `class_id` *as defined by **pci.ids***.
     fn set_class_name(&mut self) {
+        // This function sets the PCI device's `class_name` associated
+        // with its `class_id` *as defined by **pci.ids***.
         if !&self.class_id.is_empty() {
             self.class_name = match &self.class_id[..] {
                 "00" => DeviceClass::Unclassified.to_string(),
@@ -214,6 +230,37 @@ impl Properties for LinuxPCIDevice {
     fn set_device_name(&mut self, name: String) {
         self.device_name = name;
     }
+
+    fn set_enabled(&mut self) {
+        if let Ok(mut str) = std::fs::read_to_string(&self.path.join("enable")) {
+            // This file is guaranteed to end with an EOL character, so let's remove that.
+            str.pop();
+            if let Ok(val) = str.parse::<usize>() {
+                match val {
+                    0 => self.enabled = false,
+                    _ => self.enabled = true,
+                }
+            }
+        }
+    }
+}
+
+impl Default for LinuxPCIDevice {
+    fn default() -> Self {
+        LinuxPCIDevice {
+            path: PathBuf::new(),
+            address: String::new(),
+            class_id: String::new(),
+            class_name: String::new(),
+            vendor_id: String::new(),
+            vendor_name: String::new(),
+            device_id: String::new(),
+            device_name: String::new(),
+            numa_node: -1,
+            enabled: false,
+            revision: String::new(),
+        }
+    }
 }
 
 impl std::fmt::Display for LinuxPCIDevice {
@@ -242,7 +289,7 @@ impl Fetch for LinuxPCIDevice {
                 devices.push(device);
             }
         }
-        
+
         return devices;
     }
 
@@ -255,6 +302,8 @@ impl Fetch for LinuxPCIDevice {
                 let new_name = &whole_name[start_bytes + 1..end_bytes];
                 gpu.set_device_name(new_name.to_string());
             }
+
+            gpu.set_vendor_name(gpu.vendor_name().replace(" Corporation", ""));
         }
         gpus
     }
@@ -262,23 +311,22 @@ impl Fetch for LinuxPCIDevice {
 
 #[cfg(test)]
 mod tests {
-    // Note this useful idiom: importing names from outer (for mod tests) scope.
     use super::*;
 
     const PLACEHOLDER_PCI_DEVICE: &str = "/sys/bus/pci/devices/0000:00:00.0";
 
-    #[test]
-    fn test_address() {
-        let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
-        assert_ne!(device.address(), "");
-    }
-
     #[test]
     fn test_path() {
         let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
         assert_ne!(device.path(), PathBuf::new());
     }
 
+    #[test]
+    fn test_address() {
+        let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
+        assert_ne!(device.address(), "");
+    }
+
     #[test]
     fn test_class_name() {
         let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
@@ -320,4 +368,25 @@ mod tests {
         let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
         assert_ne!(device.numa_node().to_string(), "");
     }
+
+    #[test]
+    fn test_revision() {
+        let device = LinuxPCIDevice::new(PLACEHOLDER_PCI_DEVICE);
+        assert_ne!(device.revision(), "");
+    }
+
+    #[test]
+    fn test_enabled() {
+        // We can't know for sure which devices are/aren't enabled, but we
+        // could perform a test to make sure that at least one device is.
+        let devices = LinuxPCIDevice::fetch();
+        let mut enabled = false;
+        for device in devices {
+            if device.enabled {
+                enabled = true;
+            }
+        }
+
+        assert_eq!(enabled, true);
+    }
 }
